@@ -2,37 +2,42 @@
 
 
 mod os;
+mod webdav;
 mod content_encoding;
 
 use base64;
 use std::path::Path;
 use percent_encoding;
+use walkdir::WalkDir;
 use std::borrow::Cow;
 use rfsapi::RawFileData;
-use std::{cmp, f64, fmt};
-use iron::headers::UserAgent;
+use std::{cmp, f64, str};
+use std::time::SystemTime;
 use std::collections::HashMap;
 use time::{self, Duration, Tm};
 use iron::{mime, Headers, Url};
-use std::io::{BufReader, BufRead};
 use base64::display::Base64Display;
-use std::fs::{self, FileType, File};
-use iron::headers::{HeaderFormat, Header};
+use std::fmt::{self, Write as FmtWrite};
 use iron::error::HttpResult as HyperResult;
+use std::fs::{self, FileType, Metadata, File};
+use iron::headers::{HeaderFormat, UserAgent, Header};
 use mime_guess::{guess_mime_type_opt, get_mime_type_str};
+use xml::name::{OwnedName as OwnedXmlName, Name as XmlName};
+use std::io::{ErrorKind as IoErrorKind, BufReader, BufRead, Result as IoResult, Error as IoError};
 
 pub use self::os::*;
+pub use self::webdav::*;
 pub use self::content_encoding::*;
 
 
 /// The generic HTML page to use as response to errors.
-pub static ERROR_HTML: &'static str = include_str!("../../assets/error.html");
+pub static ERROR_HTML: &str = include_str!("../../assets/error.html");
 
 /// The HTML page to use as template for a requested directory's listing.
-pub static DIRECTORY_LISTING_HTML: &'static str = include_str!("../../assets/directory_listing.html");
+pub static DIRECTORY_LISTING_HTML: &str = include_str!("../../assets/directory_listing.html");
 
 /// The HTML page to use as template for a requested directory's listing for mobile devices.
-pub static MOBILE_DIRECTORY_LISTING_HTML: &'static str = include_str!("../../assets/directory_listing_mobile.html");
+pub static MOBILE_DIRECTORY_LISTING_HTML: &str = include_str!("../../assets/directory_listing_mobile.html");
 
 lazy_static! {
     /// Collection of data to be injected into generated responses.
@@ -45,28 +50,47 @@ lazy_static! {
         ass.insert("dir_icon",
             Cow::Owned(format!("data:{};base64,{}",
                                get_mime_type_str("gif").unwrap(),
-                               Base64Display::with_config(&include_bytes!("../../assets/icons/directory_icon.gif")[..], base64::STANDARD))));
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/directory.gif")[..], base64::STANDARD))));
         ass.insert("file_icon",
             Cow::Owned(format!("data:{};base64,{}",
                                get_mime_type_str("gif").unwrap(),
-                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_icon.gif")[..], base64::STANDARD))));
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/file.gif")[..], base64::STANDARD))));
         ass.insert("file_binary_icon",
             Cow::Owned(format!("data:{};base64,{}",
                                get_mime_type_str("gif").unwrap(),
-                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_binary_icon.gif")[..], base64::STANDARD))));
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_binary.gif")[..], base64::STANDARD))));
         ass.insert("file_image_icon",
             Cow::Owned(format!("data:{};base64,{}",
                                get_mime_type_str("gif").unwrap(),
-                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_image_icon.gif")[..], base64::STANDARD))));
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_image.gif")[..], base64::STANDARD))));
         ass.insert("file_text_icon",
             Cow::Owned(format!("data:{};base64,{}",
                                get_mime_type_str("gif").unwrap(),
-                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_text_icon.gif")[..], base64::STANDARD))));
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/file_text.gif")[..], base64::STANDARD))));
         ass.insert("back_arrow_icon",
             Cow::Owned(format!("data:{};base64,{}",
                                get_mime_type_str("gif").unwrap(),
-                               Base64Display::with_config(&include_bytes!("../../assets/icons/back_arrow_icon.gif")[..], base64::STANDARD))));
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/back_arrow.gif")[..], base64::STANDARD))));
+        ass.insert("new_dir_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/new_directory.gif")[..], base64::STANDARD))));
+        ass.insert("delete_file_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("png").unwrap(),
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/delete_file.png")[..], base64::STANDARD))));
+        ass.insert("rename_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("png").unwrap(),
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/rename.png")[..], base64::STANDARD))));
+        ass.insert("confirm_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("png").unwrap(),
+                               Base64Display::with_config(&include_bytes!("../../assets/icons/confirm.png")[..], base64::STANDARD))));
         ass.insert("date", Cow::Borrowed(include_str!("../../assets/date.js")));
+        ass.insert("manage", Cow::Borrowed(include_str!("../../assets/manage.js")));
+        ass.insert("manage_mobile", Cow::Borrowed(include_str!("../../assets/manage_mobile.js")));
+        ass.insert("manage_desktop", Cow::Borrowed(include_str!("../../assets/manage_desktop.js")));
         ass.insert("upload", Cow::Borrowed(include_str!("../../assets/upload.js")));
         ass.insert("adjust_tz", Cow::Borrowed(include_str!("../../assets/adjust_tz.js")));
         ass
@@ -80,10 +104,10 @@ pub static PORT_SCAN_LOWEST: u16 = 8000;
 pub static PORT_SCAN_HIGHEST: u16 = 9999;
 
 /// The app name and version to use with User-Agent or Server response header.
-pub static USER_AGENT: &'static str = concat!("http/", env!("CARGO_PKG_VERSION"));
+pub static USER_AGENT: &str = concat!("http/", env!("CARGO_PKG_VERSION"));
 
-/// Index file extensions to look for if `-i` was not specified.
-pub static INDEX_EXTENSIONS: &'static [&'static str] = &["html", "htm", "shtml"];
+/// Index file extensions to look for if `-i` was not specified and strippable extensions to look for if `-x` was specified.
+pub static INDEX_EXTENSIONS: &[&str] = &["html", "htm", "shtml"];
 
 
 /// The [WWW-Authenticate header](https://tools.ietf.org/html/rfc7235#section-4.1), without parsing.
@@ -109,6 +133,70 @@ impl HeaderFormat for WwwAuthenticate {
     }
 }
 
+#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
+pub struct CommaList<D: fmt::Display, I: Iterator<Item = D>>(pub I);
+
+impl<D: fmt::Display, I: Iterator<Item = D> + Clone> fmt::Display for CommaList<D, I> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let mut itr = self.0.clone();
+        if let Some(item) = itr.next() {
+            item.fmt(f)?;
+
+            for item in itr {
+                f.write_str(", ")?;
+                item.fmt(f)?;
+            }
+        }
+
+        Ok(())
+    }
+}
+
+#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
+pub struct DisplayThree<Df: fmt::Display, Ds: fmt::Display, Dt: fmt::Display>(pub Df, pub Ds, pub Dt);
+
+impl<Df: fmt::Display, Ds: fmt::Display, Dt: fmt::Display> fmt::Display for DisplayThree<Df, Ds, Dt> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        self.0.fmt(f)?;
+        self.1.fmt(f)?;
+        self.2.fmt(f)?;
+        Ok(())
+    }
+}
+
+
+/// `xml`'s `OwnedName::borrow()` returns a value not a reference, so it cannot be used with the libstd `Borrow` trait
+pub trait BorrowXmlName<'n> {
+    fn borrow_xml_name(&'n self) -> XmlName<'n>;
+}
+
+impl<'n> BorrowXmlName<'n> for XmlName<'n> {
+    #[inline(always)]
+    fn borrow_xml_name(&'n self) -> XmlName<'n> {
+        *self
+    }
+}
+
+impl<'n> BorrowXmlName<'n> for OwnedXmlName {
+    #[inline(always)]
+    fn borrow_xml_name(&'n self) -> XmlName<'n> {
+        self.borrow()
+    }
+}
+
+#[derive(Debug, Copy, Clone, Hash, PartialOrd, Ord, PartialEq, Eq)]
+pub struct Spaces(pub usize);
+
+impl fmt::Display for Spaces {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        for _ in 0..self.0 {
+            f.write_char(' ')?;
+        }
+        Ok(())
+    }
+}
+
+
 
 /// Uppercase the first character of the supplied string.
 ///
@@ -128,6 +216,30 @@ pub fn uppercase_first(s: &str) -> String {
     }
 }
 
+/// Percent-encode the last character if it's white space
+///
+/// Firefox treats, e.g. `href="http://henlo/menlo   "` as `href="http://henlo/menlo"`,
+/// but that final whitespace is significant, so this turns it into `href="http://henlo/menlo  %20"`
+pub fn encode_tail_if_trimmed(mut s: String) -> String {
+    let c = s.chars().rev().next();
+    if c.map(|c| c.is_whitespace()).unwrap_or(false) {
+        let c = c.unwrap();
+
+        s.pop();
+        s.push('%');
+
+        let mut cb = [0u8; 4];
+        c.encode_utf8(&mut cb);
+        for b in cb.iter().take(c.len_utf8()) {
+            write!(s, "{:02X}", b).expect("Couldn't allocate two more characters?");
+        }
+
+        s
+    } else {
+        s
+    }
+}
+
 /// Check if the specified file is to be considered "binary".
 ///
 /// Basically checks is a file is UTF-8.
@@ -143,7 +255,10 @@ pub fn uppercase_first(s: &str) -> String {
 /// assert!(!file_binary("Cargo.toml"));
 /// ```
 pub fn file_binary<P: AsRef<Path>>(path: P) -> bool {
-    let path = path.as_ref();
+    file_binary_impl(path.as_ref())
+}
+
+fn file_binary_impl(path: &Path) -> bool {
     path.metadata()
         .map(|m| is_device(&m.file_type()) || File::open(path).and_then(|f| BufReader::new(f).read_line(&mut String::new())).is_err())
         .unwrap_or(true)
@@ -203,8 +318,37 @@ pub fn percent_decode(s: &str) -> Option<Cow<str>> {
 }
 
 /// Get the timestamp of the file's last modification as a `time::Tm` in UTC.
-pub fn file_time_modified(f: &Path) -> Tm {
-    match f.metadata().expect("Failed to get file metadata").modified().expect("Failed to get file last modified date").elapsed() {
+pub fn file_time_modified_p(f: &Path) -> Tm {
+    file_time_modified(&f.metadata().expect("Failed to get file metadata"))
+}
+
+/// Get the timestamp of the file's last modification as a `time::Tm` in UTC.
+pub fn file_time_created_p(f: &Path) -> Tm {
+    file_time_created(&f.metadata().expect("Failed to get file metadata"))
+}
+
+/// Get the timestamp of the file's last access as a `time::Tm` in UTC.
+pub fn file_time_accessed_p(f: &Path) -> Tm {
+    file_time_accessed(&f.metadata().expect("Failed to get file metadata"))
+}
+
+/// Get the timestamp of the file's last modification as a `time::Tm` in UTC.
+pub fn file_time_modified(m: &Metadata) -> Tm {
+    file_time_impl(m.modified().expect("Failed to get file last modified date"))
+}
+
+/// Get the timestamp of the file's last modification as a `time::Tm` in UTC.
+pub fn file_time_created(m: &Metadata) -> Tm {
+    file_time_impl(m.created().expect("Failed to get file created date"))
+}
+
+/// Get the timestamp of the file's last access as a `time::Tm` in UTC.
+pub fn file_time_accessed(m: &Metadata) -> Tm {
+    file_time_impl(m.accessed().expect("Failed to get file accessed date"))
+}
+
+fn file_time_impl(time: SystemTime) -> Tm {
+    match time.elapsed() {
         Ok(dur) => time::now_utc() - Duration::from_std(dur).unwrap(),
         Err(ste) => time::now_utc() + Duration::from_std(ste.duration()).unwrap(),
     }
@@ -270,7 +414,7 @@ pub fn is_descendant_of<Pw: AsRef<Path>, Po: AsRef<Path>>(who: Pw, of_whom: Po)
 
 /// Check if the specified path is a direct descendant (or an equal) of the specified path, without without requiring it to
 /// exist in the first place.
-pub fn is_nonexistant_descendant_of<Pw: AsRef<Path>, Po: AsRef<Path>>(who: Pw, of_whom: Po) -> bool {
+pub fn is_nonexistent_descendant_of<Pw: AsRef<Path>, Po: AsRef<Path>>(who: Pw, of_whom: Po) -> bool {
     let mut who = fs::canonicalize(&who).unwrap_or_else(|_| who.as_ref().to_path_buf());
     let of_whom = if let Ok(p) = fs::canonicalize(of_whom) {
         p
@@ -327,6 +471,13 @@ pub fn client_mobile(hdr: &Headers) -> bool {
     hdr.get::<UserAgent>().map(|s| s.contains("Mobi") || s.contains("mobi")).unwrap_or(false)
 }
 
+/// Check if, given the request headers, the client should be treated as Microsoft software.
+///
+/// Based on https://github.com/miquels/webdav-handler-rs/blob/02433c1acfccd848a7de26889f6857cbad559076/src/handle_props.rs#L529
+pub fn client_microsoft(hdr: &Headers) -> bool {
+    hdr.get::<UserAgent>().map(|s| s.contains("Microsoft") || s.contains("microsoft")).unwrap_or(false)
+}
+
 /// Get the suffix for the icon to use to represent the given file.
 pub fn file_icon_suffix<P: AsRef<Path>>(f: P, is_file: bool) -> &'static str {
     if is_file {
@@ -347,7 +498,11 @@ pub fn file_icon_suffix<P: AsRef<Path>>(f: P, is_file: bool) -> &'static str {
 ///
 /// The specified path must point to a file.
 pub fn get_raw_fs_metadata<P: AsRef<Path>>(f: P) -> RawFileData {
-    let f = f.as_ref();
+    get_raw_fs_metadata_impl(f.as_ref())
+}
+
+fn get_raw_fs_metadata_impl(f: &Path) -> RawFileData {
+    let meta = f.metadata().expect("Failed to get requested file metadata");
     RawFileData {
         mime_type: guess_mime_type_opt(f).unwrap_or_else(|| if file_binary(f) {
             "application/octet-stream".parse().unwrap()
@@ -355,8 +510,63 @@ pub fn get_raw_fs_metadata<P: AsRef<Path>>(f: P) -> RawFileData {
             "text/plain".parse().unwrap()
         }),
         name: f.file_name().unwrap().to_str().expect("Failed to get requested file name").to_string(),
-        last_modified: file_time_modified(f),
-        size: f.metadata().expect("Failed to get requested file metadata").len(),
+        last_modified: file_time_modified(&meta),
+        size: file_length(&meta, &f),
         is_file: true,
     }
 }
+
+/// Recursively copy a directory
+///
+/// Stolen from https://github.com/mdunsmuir/copy_dir/blob/0.1.2/src/lib.rs
+pub fn copy_dir(from: &Path, to: &Path) -> IoResult<Vec<(IoError, String)>> {
+    macro_rules! push_error {
+        ($vec:ident, $path:ident, $expr:expr) => {
+            match $expr {
+                Ok(_) => (),
+                Err(e) => $vec.push((e, $path.to_string_lossy().into_owned())),
+            }
+        };
+    }
+
+    let mut errors = Vec::new();
+
+    fs::create_dir(&to)?;
+
+    // The approach taken by this code (i.e. walkdir) will not gracefully
+    // handle copying a directory into itself, so we're going to simply
+    // disallow it by checking the paths. This is a thornier problem than I
+    // wish it was, and I'd like to find a better solution, but for now I
+    // would prefer to return an error rather than having the copy blow up
+    // in users' faces. Ultimately I think a solution to this will involve
+    // not using walkdir at all, and might come along with better handling
+    // of hard links.
+    if from.canonicalize().and_then(|fc| to.canonicalize().map(|tc| (fc, tc))).map(|(fc, tc)| tc.starts_with(fc))? {
+        fs::remove_dir(&to)?;
+
+        return Err(IoError::new(IoErrorKind::Other, "cannot copy to a path prefixed by the source path"));
+    }
+
+    for entry in WalkDir::new(&from).min_depth(1).into_iter().flatten() {
+        let source_metadata = match entry.metadata() {
+            Ok(md) => md,
+            Err(err) => {
+                errors.push((err.into(), entry.path().to_string_lossy().into_owned()));
+                continue;
+            }
+        };
+
+        let relative_path = entry.path().strip_prefix(&from).expect("strip_prefix failed; this is a probably a bug in copy_dir");
+
+        let target_path = to.join(relative_path);
+
+        if !is_actually_file(&source_metadata.file_type()) {
+            push_error!(errors, relative_path, fs::create_dir(&target_path));
+            push_error!(errors, relative_path, fs::set_permissions(&target_path, source_metadata.permissions()));
+        } else {
+            push_error!(errors, relative_path, fs::copy(entry.path(), &target_path));
+        }
+    }
+
+    Ok(errors)
+}
